@@ -1,7 +1,7 @@
 /* -*- indent-tabs-mode: t; tab-width: 8; c-basic-offset: 8; -*- */
 
 /* Copyright (c) 2004 - 2006 Derek Foreman, Ben Jansens
-   Copyright (c) 2006 - 2011 Thomas Schmitt <scdbackup@gmx.net>
+   Copyright (c) 2006 - 2013 Thomas Schmitt <scdbackup@gmx.net>
    Provided under GPL version 2 or later.
 */
 
@@ -404,14 +404,14 @@ static int sgio_log_cmd(unsigned char *cmd, int cmd_len, FILE *fp_in, int flag)
 /* ts B11110 */
 static int sgio_log_reply(unsigned char *opcode, int data_dir,
                           unsigned char *data, int dxfer_len,
-                          void *fp_in, unsigned char sense[18],
-                          int sense_len, int duration, int flag)
+                          void *fp_in, unsigned char sense[18], int sense_len,
+                          double duration, int flag)
 {
 	int ret;
 
 	ret = scsi_log_reply(opcode, data_dir, data, dxfer_len, fp_in,
-	                     sense, sense_len, duration, flag);
-	return 1;
+	                   sense, sense_len, duration, flag);
+	return ret;
 }
 
 
@@ -420,6 +420,7 @@ static int sgio_test(int fd)
 	unsigned char test_ops[] = { 0, 0, 0, 0, 0, 0 };
 	sg_io_hdr_t s;
 	int ret;
+	double c_start_time, c_end_time;
 
 	memset(&s, 0, sizeof(sg_io_hdr_t));
 	s.interface_id = 'S';
@@ -430,10 +431,12 @@ static int sgio_test(int fd)
 
 	sgio_log_cmd(s.cmdp, s.cmd_len, NULL, 0);
 
+	c_start_time = burn_get_time(0);
 	ret= ioctl(fd, SG_IO, &s);
+	c_end_time = burn_get_time(0);
 
-	sgio_log_reply(s.cmdp, NO_TRANSFER, NULL, 0,
-                       NULL, s.sbp, s.sb_len_wr, s.duration, 0);
+	sgio_log_reply(s.cmdp, NO_TRANSFER, NULL, 0, NULL,
+	               s.sbp, s.sb_len_wr, c_end_time - c_start_time, 0);
 	return ret;
 }
 
@@ -446,6 +449,7 @@ static int sgio_inquiry_cd_drive(int fd, char *fname)
 	unsigned char *sense = NULL;
 	char *msg = NULL, *msg_pt;
 	int ret = 0, i;
+	double c_start_time, c_end_time;
 
 	BURN_ALLOC_MEM(buf, struct buffer, 1);
 	BURN_ALLOC_MEM(sense, unsigned char, 128);
@@ -465,7 +469,9 @@ static int sgio_inquiry_cd_drive(int fd, char *fname)
 
 	sgio_log_cmd(s.cmdp, s.cmd_len, NULL, 0);
 
+	c_start_time = burn_get_time(0);
 	ret = ioctl(fd, SG_IO, &s);
+	c_end_time = burn_get_time(0);
 	if (ret == -1) {
 		sprintf(msg,
 			 "INQUIRY on '%s' : ioctl(SG_IO) failed , errno= %d",
@@ -476,8 +482,8 @@ static int sgio_inquiry_cd_drive(int fd, char *fname)
 		goto ex;
 	}
 
-	sgio_log_reply(s.cmdp, FROM_DRIVE, buf->data, s.dxfer_len,
-                       NULL, s.sbp, s.sb_len_wr, s.duration, 0);
+	sgio_log_reply(s.cmdp, FROM_DRIVE, buf->data, s.dxfer_len, NULL,
+                       s.sbp, s.sb_len_wr, c_end_time - c_start_time, 0);
 
 	if (s.sb_len_wr > 0 || s.host_status != Libburn_sg_host_oK ||
 	    s.driver_status != Libburn_sg_driver_oK) {
@@ -686,10 +692,15 @@ static int sg_fcntl_lock(int *fd, char *fd_name, int l_type, int verbous)
 
 
 /* ts A60926 */
+/* @param scan_mode  0= open for drivce aquiration
+                     1= open for scanning with guessed names
+                     2= open for scanning with /proc/sys/dev/cdrom/info names
+*/
 static int sg_open_drive_fd(char *fname, int scan_mode)
 {
-	int open_mode = O_RDWR, fd, tries= 0;
+	int open_mode = O_RDWR, fd, tries= 0, is_std_adr, report_as_note = 0;
 	char msg[81];
+	struct stat stbuf;
 
 	/* ts A70409 : DDLP-B */
 	/* >>> obtain single lock on fname */
@@ -746,12 +757,26 @@ try_open:;
 			return -1;
 			
 		}
-		if (scan_mode)
-			return -1;
 		sprintf(msg, "Failed to open device '%s'",fname);
-		libdax_msgs_submit(libdax_messenger, -1, 0x00020005,
+		if (scan_mode) {
+			is_std_adr = (strncmp(fname, "/dev/sr", 7) == 0 ||
+			              strncmp(fname, "/dev/scd", 8) == 0);
+			if(scan_mode == 1 && is_std_adr &&
+			   stat(fname, &stbuf) != -1)
+				report_as_note = 1;
+			else if(scan_mode == 2 && (!is_std_adr) &&
+			        stat(fname, &stbuf) != -1)
+				report_as_note = 1;
+			if (report_as_note)
+				libdax_msgs_submit(libdax_messenger, -1,
+				   0x0002000e,
+				   LIBDAX_MSGS_SEV_NOTE, LIBDAX_MSGS_PRIO_HIGH,
+				   msg, errno, 0);
+		} else {
+			libdax_msgs_submit(libdax_messenger, -1, 0x00020005,
 				LIBDAX_MSGS_SEV_SORRY, LIBDAX_MSGS_PRIO_HIGH,
 				msg, errno, 0);
+		}
 		return -1;
 	}
 	sg_fcntl_lock(&fd, fname, F_WRLCK, 1);
@@ -1080,11 +1105,12 @@ ex:;
 }	
 
 
+/* @param flag bit0= do not complain about failure to open /dev/sr /dev/scd */
 static int sg_open_for_enumeration(char *fname, int flag)
 {
 	int fd;
 
-	fd = sg_open_drive_fd(fname, 1);
+	fd = sg_open_drive_fd(fname, 1 + (flag & 1));
 	if (fd < 0) {
 		if (linux_sg_enumerate_debug || linux_ata_enumerate_verbous)
 			fprintf(stderr, "open failed, errno=%d  '%s'\n",
@@ -1225,6 +1251,7 @@ static int fname_drive_is_listed(char *fname, int flag)
 /* ts A80731 : Directly open the given address.
    @param flag bit0= do not complain about missing file
                bit1= do not check whether drive is already listed
+               bit2= do not complain about failure to open /dev/sr /dev/scd
 */
 static int fname_enumerate(char *fname, int flag)
 {
@@ -1247,7 +1274,7 @@ static int fname_enumerate(char *fname, int flag)
 		{ret = -1; goto ex;}
 	}
 
-	fd = sg_open_for_enumeration(fname, 0);
+	fd = sg_open_for_enumeration(fname, !!(flag & 4));
 	if (fd < 0)
 		{ret = 0; goto ex;}
 	is_ata = is_ata_drive(fname, fd);
@@ -1399,7 +1426,7 @@ static int add_proc_info_drives(int flag)
 	for (i = 0; i < list_count; i++) {
 		if (burn_drive_is_banned(list[i]))
 	continue;
-		ret = fname_enumerate(list[i], 1);
+		ret = fname_enumerate(list[i], 1 | 4);
 		if (ret == 1)
 			count++;
 	}
@@ -1857,6 +1884,167 @@ int sg_release(struct burn_drive *d)
 	return 0;
 }
 
+/* @return -1= transport failed, give up drive
+            0= transport failed, do not retry
+            1= transport succeeded
+            2- transport failed, please retry
+*/
+static int evaluate_transport_success(struct burn_drive *d, struct command *c,
+                                      FILE *fp,
+                                      unsigned short host_status,
+                                      unsigned short driver_status)
+{
+	int ret, do_retry= 0, give_up_drive= 0, sev;
+	char *msg = NULL, *host_problem, *driver_problem, *driver_sugg;
+
+	BURN_ALLOC_MEM(msg, char, 161);
+
+	if ((host_status == Libburn_sg_host_oK &&
+             (driver_status & 0xf7) == Libburn_sg_driver_oK) || c->error)
+		{ret = 1; goto ex;} /* No transport problems */
+
+	/* See http://www.tldp.org/HOWTO/SCSI-Generic-HOWTO/x291.html */
+
+	switch(host_status) {
+	case 0x00:
+		host_problem =
+		"SG_ERR_DID_OK (No error)";
+	break; case 0x01:
+		host_problem =
+		"SG_ERR_DID_NO_CONNECT (Could not connect before timeout period)";
+		give_up_drive= 1;
+	break; case 0x02:
+		host_problem =
+		"SG_ERR_DID_BUS_BUSY (Bus stayed busy through time out period)";
+	break; case 0x03:
+		host_problem =
+		"SG_ERR_DID_TIME_OUT (Timed out for miscellaneous reasons)";
+	break; case 0x04:
+		host_problem =
+		"SG_ERR_DID_BAD_TARGET (Bad target, device not responding ?)";
+		give_up_drive= 1;
+	break; case 0x05:
+		host_problem =
+		"SG_ERR_DID_ABORT (Told to abort)";
+	break; case 0x06:
+		host_problem =
+		"SG_ERR_DID_PARITY (Parity error)";
+	break; case 0x07:
+		host_problem =
+		"SG_ERR_DID_ERROR (Internal error detected in the host adapter)";
+		give_up_drive= 1;
+	break; case 0x08:
+		host_problem =
+		"SG_ERR_DID_RESET (The SCSI bus or the device have been reset)";
+		give_up_drive= 1;
+	break; case 0x09:
+		host_problem =
+		"SG_ERR_DID_BAD_INTR (Got an unexpected interrupt)";
+	break; case 0x0a:
+		host_problem =
+		"SG_ERR_DID_PASSTHROUGH (Force command past mid-layer)";
+	break; case 0x0b:
+		host_problem =
+		"SG_ERR_DID_SOFT_ERROR (The low level driver wants a retry)";
+		do_retry = 1;
+	break; default:
+		host_problem =
+		"? (unknown host_status code)";
+	}
+	if (host_status != Libburn_sg_host_oK) {
+		sprintf(msg, "SCSI command %2.2Xh yielded host problem: ",
+			(unsigned int) c->opcode[0]);
+		sprintf(msg+strlen(msg), "0x%x %s",
+			(unsigned int) host_status, host_problem);
+		sev = LIBDAX_MSGS_SEV_FAILURE;
+		if (do_retry && !give_up_drive)
+			sev = LIBDAX_MSGS_SEV_DEBUG;
+		libdax_msgs_submit(libdax_messenger, d->global_index,
+			0x000201a7, sev, LIBDAX_MSGS_PRIO_HIGH, msg, 0, 0);
+		sprintf(msg, "--- SG_IO: host_status= 0x%x %s",
+			(unsigned int) host_status, host_problem);
+		scsi_log_message(d, fp, msg, 0);
+	}
+
+	switch (driver_status & 0x07) {
+	case 0:
+		driver_problem = "SG_ERR_DRIVER_OK";
+	break; case 1:
+		driver_problem = "SG_ERR_DRIVER_BUSY";
+	break; case 2:
+		driver_problem = "SG_ERR_DRIVER_SOFT";
+	break; case 3:
+		driver_problem = "SG_ERR_DRIVER_MEDIA";
+	break; case 4:
+		driver_problem = "SG_ERR_DRIVER_ERROR";
+	break; case 5:
+		driver_problem = "SG_ERR_DRIVER_INVALID";
+	break; case 6:
+		driver_problem = "SG_ERR_DRIVER_TIMEOUT";
+	break; case 7:
+		driver_problem = "SG_ERR_DRIVER_HARD";
+	break; default:
+		driver_problem = "(unknown driver_status code)";
+	}
+	switch (driver_status & 0xf0) {
+	case 0:
+		driver_sugg = "(no suggestion)";
+	break; case 0x10:
+		driver_sugg = "SG_ERR_SUGGEST_RETRY";
+		do_retry = 1;
+	break; case 0x20:
+		driver_sugg = "SG_ERR_SUGGEST_ABORT";
+		give_up_drive= 1;
+	break; case 0x30:
+		driver_sugg = "SG_ERR_SUGGEST_REMAP";
+		give_up_drive= 1;
+	break; case 0x40:
+		driver_sugg = "SG_ERR_SUGGEST_DIE";
+		give_up_drive= 1;
+	break; case 0x80:
+		driver_sugg = "SG_ERR_SUGGEST_SENSE";
+	default:
+		driver_sugg = "(unknown driver_status suggestion)";
+	}
+	if ((driver_status & 0xf7) != Libburn_sg_driver_oK) {
+		sprintf(msg, "SCSI command %2.2Xh yielded driver problem: ",
+			(unsigned int) c->opcode[0]);
+		sprintf(msg+strlen(msg), "driver_status= 0x%x %s / %s",
+			(unsigned int) driver_status,
+			driver_problem, driver_sugg);
+		sev = LIBDAX_MSGS_SEV_FAILURE;
+		if (do_retry && !give_up_drive)
+			sev = LIBDAX_MSGS_SEV_DEBUG;
+		libdax_msgs_submit(libdax_messenger, d->global_index,
+			0x000201a8, sev, LIBDAX_MSGS_PRIO_HIGH, msg, 0, 0);
+		sprintf(msg, "--- SG_IO: driver_status= 0x%x %s / %s",
+			(unsigned int) driver_status,
+			driver_problem, driver_sugg);
+		scsi_log_message(d, fp, msg, 0);
+	}
+
+	if (! do_retry)
+		c->error = 1;
+	ret = give_up_drive ? -1 : do_retry ? 2 : 0;
+ex:;
+	BURN_FREE_MEM(msg);
+	return ret;
+}
+
+static void react_on_drive_loss(struct burn_drive *d, struct command *c,
+                                FILE *fp)
+{
+	sg_close_drive(d);
+	d->released = 1;
+	d->busy = BURN_DRIVE_IDLE;
+	d->cancel = 1;
+	c->error = 1;
+	libdax_msgs_submit(libdax_messenger,
+				 d->global_index, 0x000201a6,
+				 LIBDAX_MSGS_SEV_FATAL, LIBDAX_MSGS_PRIO_HIGH,
+				 "Lost connection to drive", 0, 0);
+	scsi_log_message(d, fp, "--- SG_IO: Gave up connection to drive", 0);
+}
 
 /** Sends a SCSI command to the drive, receives reply and evaluates wether
     the command succeeded or shall be retried or finally failed.
@@ -1877,6 +2065,9 @@ int sg_issue_command(struct burn_drive *d, struct command *c)
 
 	BURN_ALLOC_MEM(msg, char, 161);
 
+	c->error = 0;
+	memset(c->sense, 0, sizeof(c->sense));
+
 	/* <<< ts A60821
 	   debug: for tracing calls which might use open drive fds */
 	sprintf(msg, "sg_issue_command   d->fd= %d  d->released= %d\n",
@@ -1893,8 +2084,6 @@ int sg_issue_command(struct burn_drive *d, struct command *c)
 			    "\n-----------------------------------------\n");
 		}
 	}
-	if (burn_sg_log_scsi & 3)
-		scsi_log_cmd(c,fp,0);
 
 	/* ts A61010 : with no fd there is no chance to send an ioctl */
 	if (d->fd < 0) {
@@ -1902,8 +2091,11 @@ int sg_issue_command(struct burn_drive *d, struct command *c)
 		{ret = 0; goto ex;}
 	}
 
+
 	c->error = 0;
 	memset(&s, 0, sizeof(sg_io_hdr_t));
+	if (burn_sg_log_scsi & 3)
+		scsi_log_cmd(c,fp,0);
 
 	s.interface_id = 'S';
 
@@ -1929,8 +2121,10 @@ int sg_issue_command(struct burn_drive *d, struct command *c)
 	s.cmdp = c->opcode;
 	s.mx_sb_len = 32;
 	s.sbp = c->sense;
-	memset(c->sense, 0, sizeof(c->sense));
-	s.timeout = 200000;
+	if (c->timeout > 0)
+		s.timeout = c->timeout;
+	else
+		s.timeout = Libburn_scsi_default_timeouT;
 	if (c->page && !no_c_page) {
 		s.dxferp = c->page->data;
 		if (c->dir == FROM_DRIVE) {
@@ -1964,8 +2158,14 @@ int sg_issue_command(struct burn_drive *d, struct command *c)
 
 	start_time = time(NULL);
 	for(i = 0; !done; i++) {
+
+		memset(c->sense, 0, sizeof(c->sense));
+		c->start_time = burn_get_time(0);
+
 		err = ioctl(d->fd, SG_IO, &s);
 
+		c->end_time = burn_get_time(0);
+
 		/* ts A61010 */
 		/* a ssert(err != -1); */
 		if (err == -1) {
@@ -1974,30 +2174,31 @@ int sg_issue_command(struct burn_drive *d, struct command *c)
 				 LIBDAX_MSGS_SEV_FATAL, LIBDAX_MSGS_PRIO_HIGH,
 				 "Failed to transfer command to drive",
 				 errno, 0);
-			sg_close_drive(d);
-			d->released = 1;
-			d->busy = BURN_DRIVE_IDLE;
-			c->error = 1;
+			sprintf(msg, "--- SG_IO: return= -1 , ");
+		        sprintf(msg + strlen(msg), "errno= %d , ", errno);
+		        sprintf(msg + strlen(msg),
+			        "host_status= 0x%x , driver_status= 0x%x",
+			        (unsigned int) s.host_status,
+			        (unsigned int) s.driver_status);
+			scsi_log_message(d, fp, msg, 0);
+			react_on_drive_loss(d, c, fp);
 			{ret = -1; goto ex;}
 		}
                 done = scsi_eval_cmd_outcome(d, c, fp, s.sbp, s.sb_len_wr,
-				s.duration, start_time, s.timeout, i, 0);
+				             start_time, s.timeout, i, 0);
+		if (d->cancel)
+	break;
+		ret = evaluate_transport_success(d, c, fp,
+		                               s.host_status, s.driver_status);
+		if (ret == -1)
+			react_on_drive_loss(d, c, fp);
+		if (ret <= 0)
+			{ret = -1; goto ex;}
+		if (d->cancel)
+	break;
+		/* if ! done : loop for retry */;
 	}
 
-	if (s.host_status != Libburn_sg_host_oK || 
-	    (s.driver_status != Libburn_sg_driver_oK && !c->error)) {
-		sprintf(msg,
-			"SCSI command %2.2Xh indicates host or driver error:",
-			(unsigned int) c->opcode[0]);
-		sprintf(msg+strlen(msg),
-			" host_status= %xh , driver_status= %xh",
-			(unsigned int) s.host_status,
-			(unsigned int) s.driver_status);
-		libdax_msgs_submit(libdax_messenger, d->global_index,
-				0x0002013b,
-				LIBDAX_MSGS_SEV_DEBUG, LIBDAX_MSGS_PRIO_HIGH,
-				msg, 0, 0);
-	}
 	ret = 1;
 ex:;
 	BURN_FREE_MEM(msg);
