@@ -3,7 +3,7 @@
 
 /* xorriso - creates, loads, manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2012 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2013 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -173,6 +173,7 @@ int Xorriso_new(struct XorrisO ** xorriso,char *progname, int flag)
  m->copyright_file[0]= 0;
  m->biblio_file[0]= 0;
  m->abstract_file[0]= 0;
+ strcpy(m->application_use, " ");
  m->session_logfile[0]= 0;
  m->session_lba= -1;
  m->session_blocks= 0;
@@ -226,7 +227,9 @@ int Xorriso_new(struct XorrisO ** xorriso,char *progname, int flag)
  m->ban_stdio_write= 0;
  m->do_dummy= 0;
  m->do_close= 0;
- m->speed= 0;
+ m->auto_close= 0;
+ m->write_speed= 0;  /* max */
+ m->read_speed= -2;  /* do not set */
  m->fs= 4*512; /* 4 MiB */
  m->padding= 300*1024;
  m->do_padding_by_libisofs= 0;
@@ -294,7 +297,9 @@ int Xorriso_new(struct XorrisO ** xorriso,char *progname, int flag)
  m->do_restore_sort_lba= 0;
  m->do_strict_acl= 0;
  m->dialog= 0;
+ m->buffered_dialog= NULL;
  m->bsl_interpretation= 0;
+ m->sh_style_result= 0;
  m->search_mode= 0;
  m->structured_search= 1;
  m->do_iso_rr_pattern= 1;
@@ -307,6 +312,7 @@ int Xorriso_new(struct XorrisO ** xorriso,char *progname, int flag)
  m->hfsplus_hidings= NULL;
  m->disk_excl_mode= 1;
  m->use_stdin= 0;
+ m->tolerate_stdin_eof= 0;
  m->result_page_length= 0;
  m->result_page_width= 80;
  m->mark_text[0]= 0;
