@@ -67,6 +67,7 @@ int Xorriso_cdrskin_help(struct XorrisO *xorriso, int flag)
 "\t-toc\t\tretrieve and print TOC/PMA data",
 "\t-atip\t\tretrieve media state, print \"Is *erasable\"",
 "\t-multi\t\tgenerate a TOC that allows multi session",
+"\t--multi_if_possible\tapply -multi if the medium supports it",
 "\t-waiti\t\twait until input is available before opening SCSI",
 "\t-tao\t\tWrite disk in TAO mode.",
 "\t-dao\t\tWrite disk in SAO mode.",
@@ -108,7 +109,8 @@ int Xorriso_cdrskin(struct XorrisO *xorriso, char *whom, int argc, char **argv,
  int do_atip= 0, do_checkdrive= 0, do_eject= 0, do_scanbus= 0;
  int do_toc= 0, do_verbous= 0, do_version= 0, do_help= 0, do_waiti= 0;
  int do_multi= 0, do_msinfo= 0, do_grow= 0, do_isosize= 0, do_xa1= 0;
- double write_start_address= -1.0, tsize= -1.0;
+ int do_auto_close= 0;
+ double write_start_address= -1.0, tsize= -1.0, mem_auto_close;
  char *track_source= NULL, *dev_adr= NULL, *cpt;
  char mem_report_about_text[80], *report_about= "SORRY", blank_mode[80];
  char speed[80], *argpt;
@@ -165,6 +167,7 @@ static char blank_help[][80]= {
 };
 
  mem_do_close= xorriso->do_close;
+ mem_auto_close= xorriso->auto_close;
  Xorriso_alloc_meM(track_source, char, SfileadrL);
  Xorriso_alloc_meM(dev_adr, char, SfileadrL);
 
@@ -297,6 +300,10 @@ no_volunteer:;
      do_msinfo= 1;
    } else if(strcmp(argpt, "-multi")==0) {
      do_multi= 1;
+     do_auto_close= 0;
+   } else if(strcmp(argv[i], "--multi_if_possible") == 0) {
+     do_multi= 1;
+     do_auto_close= 1;
    } else if(strcmp(argpt, "-nopad")==0) {
      xorriso->padding= 0;
    } else if(strcmp(argv[i], "--no_rc")==0) { /* intentional: argv[i] */
@@ -510,9 +517,15 @@ no_volunteer:;
  }
  if(track_source[0]) {
    xorriso->do_close= !do_multi;
+   xorriso->auto_close= do_auto_close;
    ret= Xorriso_burn_track(xorriso, (off_t) write_start_address,
                            track_source, (off_t) tsize,
                    (!!do_grow) | ((!!do_isosize) << 1) | ((do_xa1 == 1) << 2));
+   if(ret == 2) {
+     ret= Xorriso_retry_burn_track(xorriso, (off_t) write_start_address,
+                           track_source, (off_t) tsize,
+                   (!!do_grow) | ((!!do_isosize) << 1) | ((do_xa1 == 1) << 2));
+   }
    aq_ret= Xorriso_reaquire_outdev(xorriso, 2*(ret>0));
    if(ret<=0 && ret<aq_ret)
      goto ex;
@@ -534,6 +547,7 @@ ex:;
  }
  Xorriso_option_report_about(xorriso, mem_report_about_text, 0);
  xorriso->do_close= mem_do_close;
+ xorriso->auto_close= mem_auto_close;
  Xorriso_free_meM(dev_adr);
  Xorriso_free_meM(track_source);
  return(ret);
@@ -587,7 +601,8 @@ int Xorriso_genisofs_count_args(struct XorrisO *xorriso, int argc, char **argv,
     ""
  };
  static char arg1_options[][41]= {
-    "-abstract", "-A", "-appid", "-biblio", "-b", "-B", "-boot-load-seg",
+    "-abstract", "-A", "-appid", "--application_use",
+    "-biblio", "-b", "-B", "-boot-load-seg",
     "-boot-load-size", "-C", "-c", "-check-session", "-copyright",
     "-dir-mode", "-eltorito-id", "-eltorito-selcrit",
     "-file-mode", "-G", "-gid", "-hide", "-hide-list",
@@ -619,7 +634,7 @@ int Xorriso_genisofs_count_args(struct XorrisO *xorriso, int argc, char **argv,
     "-partition_sec_hd", "-partition_cyl_align", "-untranslated_name_len",
     "-rr_reloc_dir", "-hfsplus-serial-no", "-prep-boot-part", "-efi-boot-part",
     "-hfsplus-block-size", "-apm-block-size", "--grub2-mbr",
-    "--grub2-sparc-core",
+    "--grub2-sparc-core", "--sort-weight-list", "--sort-weight-patterns",
     ""
  };
  static char arg2_options[][41]= {
@@ -676,7 +691,7 @@ int Xorriso_genisofs_ignore(struct XorrisO *xorriso, char *whom,
    "-L", "-no-bak", "-no-cache-inodes",
    "-no-split-symlink-components", "-no-split-symlink-fields", "-nobak",
    "-force-rr", "-T",
-   "-no-iso-translate", "-gui",
+   "-no-iso-translate",
    ""
  };
  static char ignored_arg1_options[][41]= {
@@ -774,6 +789,7 @@ int Xorriso_genisofs_help(struct XorrisO *xorriso, int flag)
 "  --quoted_path_list FILE     File with list of quoted pathnames to process",
 "  -print-size                 Print estimated filesystem size and exit",
 "  -quiet                      Run quietly",
+"  -gui                        Switch behaviour for GUI",
 "  -R, -rock                   Generate Rock Ridge directory information",
 "  -r, -rational-rock          Generate rationalized Rock Ridge directory information",
 "  --norock                    Disable Rock Ridge. (Strongly discouraged !)",
@@ -792,6 +808,7 @@ int Xorriso_genisofs_help(struct XorrisO *xorriso, int flag)
 "  -abstract FILE              Set Abstract filename",
 "  -biblio FILE                Set Bibliographic filename",
 "  -copyright FILE             Set Copyright filename",
+"  --application_use CHAR|PATH  Set content of Application Use field",
 "  -jigdo-jigdo FILE           Produce a jigdo .jigdo file as well as the .iso",
 "  -jigdo-template FILE        Produce a jigdo .template file as well as the .iso",
 "  -jigdo-min-file-size SIZE   Minimum size for a file to be listed in the jigdo file", 
@@ -850,6 +867,8 @@ int Xorriso_genisofs_help(struct XorrisO *xorriso, int flag)
 "  isolinux_mbr=on|auto|off    Control eventual isohybrid MBR generation",
 #endif
 "  --sort-weight NUMBER FILE   Set LBA weight number to file or file tree",
+"  --sort-weight-list DISKFILE Read list of NUMBER FILE pairs for --sort-weight",
+"  --sort-weight-patterns DISKFILE  --sort-weight-list with pattern expansion",
 "  --stdio_sync on|off|number  Control forced output to disk files",
 "  --no-emul-toc               Save 64 kB size on random access output files",
 "  --emul-toc                  Multi-session history on such output files",
@@ -876,7 +895,7 @@ int Xorriso_genisofs_help(struct XorrisO *xorriso, int flag)
  Xorriso_option_report_about(xorriso, "NOTE", 0);
  for(i= 0; strcmp(helptext[i], "@End_of_helptexT@")!=0; i++) {
    sprintf(xorriso->info_text, "%s\n", helptext[i]);
-   Xorriso_info(xorriso, 1);
+   Xorriso_info(xorriso, 1 | 2);
  }
  Xorriso_option_report_about(xorriso, ra_text, 0);
  return(1);
@@ -1143,10 +1162,10 @@ int Xorriso_genisofs(struct XorrisO *xorriso, char *whom,
  int root_seen= 0, do_md5_mem, option_d= 0, arg_count;
  mode_t mode_and, mode_or;
  int with_boot_image= 0, with_cat_path= 0, with_emul_toc= 0;
- int old_root_md5= 1, old_root_dev= 0, old_root_ino= 1;
+ int old_root_md5= 1, old_root_dev= 0, old_root_ino= 1, sort_file_pattern= 0;
  int *weight_list= NULL, weight_count= 0;
  int *delay_opt_list= NULL, delay_opt_count= 0;
- char *sfe= NULL, *adr= NULL, ra_text[80], *pathspec= NULL;
+ char *sfe= NULL, *adr= NULL, ra_text[80], *pathspec= NULL, *sort_file= NULL;
  char *ept, *add_pt, *eff_path= NULL, *indev= NULL, msc[80], *cpt;
  char *old_root= NULL, *argpt, *hargv[1];
  char *boot_path, partno_text[8], *iso_rr_pt, *disk_pt, *rpt, *wpt;
@@ -1161,6 +1180,7 @@ int Xorriso_genisofs(struct XorrisO *xorriso, char *whom,
  Xorriso_alloc_meM(eff_path, char, SfileadrL);
  Xorriso_alloc_meM(indev, char, SfileadrL+8);
  Xorriso_alloc_meM(old_root, char, SfileadrL);
+ Xorriso_alloc_meM(sort_file, char, SfileadrL);
 
  for(i= 0; i<argc; i++) {
    if(strcmp(argv[i], "-log-file") == 0 ||
@@ -1207,7 +1227,7 @@ int Xorriso_genisofs(struct XorrisO *xorriso, char *whom,
 
  if(xorriso->boot_image_cat_path[0])
    with_cat_path= -1;
- adr[0]= indev[0]= msc[0]= old_root[0]= 0;
+ adr[0]= indev[0]= msc[0]= old_root[0]= sort_file[0]= 0;
  for(i= 0; i<argc; i++) {
    ret= Xorriso_genisofs_strip_dash(xorriso, argv[i], &argpt, 0);
    if(ret <= 0)
@@ -1409,6 +1429,8 @@ int Xorriso_genisofs(struct XorrisO *xorriso, char *whom,
      xorriso->no_emul_toc|= 1;
    } else if(strcmp(argpt, "-log-file") == 0) {
      /* already handled before this loop */;
+   } else if(strcmp(argpt, "-gui") == 0) {
+     xorriso->pacifier_interval= 0.25;
    } else {
      if(argv[i][0] == '-') {
        ret= Xorriso_genisofs_fused_options(xorriso, whom, argv[i] + 1,
@@ -1659,6 +1681,7 @@ not_enough_args:;
              strcmp(argpt, "-p")==0 || strcmp(argpt, "-preparer")==0 ||
              strcmp(argpt, "-P")==0 || strcmp(argpt, "-publisher")==0 ||
              strcmp(argpt, "-A")==0 || strcmp(argpt, "-appid")==0 ||
+             strcmp(argpt, "--application_use")==0 ||
              strcmp(argpt, "-sysid")==0 ||
              strcmp(argpt, "-biblio")==0 ||
              strcmp(argpt, "-copyright")==0 ||
@@ -1687,6 +1710,8 @@ not_enough_args:;
        ret= Xorriso_option_copyright_file(xorriso, argv[i], 0);
      else if(strcmp(argpt, "-abstract")==0)
        ret= Xorriso_option_abstract_file(xorriso, argv[i], 0);
+     else if(strcmp(argpt, "--application_use")==0)
+       ret= Xorriso_option_application_use(xorriso, argv[i], 0);
      if(ret<=0)
        goto problem_handler_2;
    } else if(strcmp(argpt, "-m")==0 || strcmp(argpt, "-exclude")==0 ||
@@ -1787,6 +1812,15 @@ not_enough_args:;
      i+= 2;
      /* memorize for find runs after pathspecs have been added */
      weight_list[weight_count++]= i - 2;
+   } else if(strcmp(argpt, "--sort-weight-list") == 0 ||
+             strcmp(argpt, "--sort-weight-patterns") == 0) {
+     if(i + 1 >= argc)
+       goto not_enough_args;
+     i++;
+     if(Sfile_str(sort_file, argv[i], 0) <= 0)
+       {ret= -1; goto ex;}
+     sort_file_pattern= (strcmp(argpt, "--sort-weight-patterns") == 0);
+
    } else if(strcmp(argpt, "-z")==0 ||
              strcmp(argpt, "-transparent-compression")==0) {
      /* was already handled in first argument scan */;
@@ -2023,8 +2057,14 @@ rr_reloc_dir:;
      Xorriso_relax_compliance(xorriso, "deep_paths_off:long_paths_off", 0);
 
    } else if(strcmp(argpt, "-log-file") == 0) {
+     if(i + 1 >= argc)
+       goto not_enough_args;
+     i+= 1;
      /* was already handled before this loop */;
 
+   } else if(strcmp(argpt, "-gui") == 0) {
+     /* was already handled in first argument scan */;
+
    } else if(argpt[0]=='-' && argpt[1]!=0) {
      ret= Xorriso_genisofs_fused_options(xorriso, whom, argv[i] + 1,
                                   &option_d, &iso_level, &lower_r, ra_text, 1);
@@ -2184,6 +2224,12 @@ problem_handler_2:;
      goto ex;
  }
 
+ if(sort_file[0]) {
+   ret= Xorriso_apply_sort_file(xorriso, sort_file, sort_file_pattern);
+   if(ret<=0)
+     goto ex;
+ }
+
  for(j= 0; j < weight_count; j++) {
    i= weight_list[j];
    /* find argv[i+2] -exec sort_weight argv[i+1] */
@@ -2435,6 +2481,7 @@ ex:;
    free(weight_list);
  if(delay_opt_list != NULL)
    free(delay_opt_list);
+ Xorriso_free_meM(sort_file);
  Xorriso_free_meM(sfe);
  Xorriso_free_meM(adr);
  Xorriso_free_meM(pathspec);
