@@ -1913,10 +1913,19 @@ int Xorriso_option_clone(struct XorrisO *xorriso, char *origin, char *dest,
 }
 
 
-/* Option -close "on"|"off" */
+/* Option -close "on"|"off"|"as_needed" */
 int Xorriso_option_close(struct XorrisO *xorriso, char *mode, int flag)
 {
- xorriso->do_close= !!strcmp(mode, "off");
+ if(strcmp(mode, "off") == 0) {
+   xorriso->do_close= 0;
+   xorriso->auto_close= 0;
+ } else if(strcmp(mode, "as_needed") == 0) {
+   xorriso->do_close= 0;
+   xorriso->auto_close= 1;
+ } else {
+   xorriso->do_close= 1;
+   xorriso->auto_close= 0;
+ }
  return(1);
 }
 
@@ -1966,6 +1975,7 @@ int Xorriso_option_close_filter_list(struct XorrisO *xorriso, int flag)
 int Xorriso_option_commit(struct XorrisO *xorriso, int flag)
 {
  int ret;
+ char eternal_problem_status_text_mem[80];
 
  if(!Xorriso_change_is_pending(xorriso, 0)) {
    sprintf(xorriso->info_text, "-commit: No image modifications pending");
@@ -1979,7 +1989,14 @@ int Xorriso_option_commit(struct XorrisO *xorriso, int flag)
      {ret= 2; goto ex;}
  }
  Xorriso_process_errfile(xorriso, 0, "burn session start", 0, 1);
+ Xorriso_get_problem_status(xorriso, eternal_problem_status_text_mem, 1);
  ret= Xorriso_write_session(xorriso, 0);
+ if(ret == 2) {
+   if(Xorriso__severity_cmp("WARNING", eternal_problem_status_text_mem) > 0)
+     strcpy(eternal_problem_status_text_mem, "WARNING");
+   Xorriso_set_problem_status(xorriso, eternal_problem_status_text_mem, 1);
+   ret= Xorriso_retry_write_session(xorriso, 0);
+ }
  Xorriso_process_errfile(xorriso, 0, "burn session end", 0, 1);
  if(ret<=0)
    goto ex;
