@@ -1533,6 +1533,9 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "                 those lists empty. Defaulty entry in \"risky\" is \"/dev\".",
 "  -grow_blindly \"off\"|predicted_nwa",
 "                 Switch between modifying and blind growing.",
+"  -read_speed number[\"k/s\"|\"m/s\"|\"[x]CD\"|\"[x]DVD\"|\"[x]BD\"]",
+"                 Set the read speed. Default is \"none\" = do not set speed",
+"                 before reading.",
 "  -load \"session\"|\"track\"|\"lba\"|\"sbsector\"|\"volid\"|\"auto\" id",
 "                 Load a particular (outdated) ISO image from a -dev or",
 "                 -indev which hosts more than one session.",
@@ -1607,6 +1610,10 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "                 Specifies the name of the Bibliographic File. (37 chars)",
 "  -abstract_file name",
 "                 Specifies the name of the Abstract File. (37 chars)",
+"  -application_use character|0xXY|disk_path",
+"                 Specifies the content of Application Use field. (512 bytes)",
+"                 A single character or a hex code gets repeated 512 times.",
+"                 Other text gets opened as data file and 512 bytes are read.",
 "  -joliet \"on\"|\"off\"",
 "                 Generate Joliet info additional to Rock Ridge info.",
 "  -hfsplus \"on\"|\"off\"",
@@ -1893,14 +1900,14 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "                 Give up any unejected drive afterwards.",
 "  -write_type \"auto\"|\"tao\"|\"sao/dao\"",
 "                 Set write type for CD-R[W], DVD-R[W], DVD+R, BD-R.",
-"  -close \"on\"|\"off\"",
+"  -close \"on\"|\"off\"|\"as_needed\"",
 "                 If \"on\" then mark the written medium as not appendable.",
 "  -padding number[\"k\"|\"m\"]|\"included\"|\"appended\"",
 "                 Append extra bytes to image stream. (Default is 300k)",
 "  -dummy \"on\"|\"off\"",
 "                 If \"on\" simulate burning. Refuse if medium cannot simulate.",
 "  -speed number[\"k/s\"|\"m/s\"|\"[x]CD\"|\"[x]DVD\"|\"[x]BD\"]",
-"                 Set the burn speed. Default is 0 = maximum speed.",
+"                 Set the burn speed. Default is \"max\" = maximum speed.",
 "  -stream_recording \"on\"|\"off\"",
 "                 Try to circumvent slow checkread on DVD-RAM, BD-RE, BD-R.",
 "  -dvd_obs \"default\"|\"32k\"|\"64k\"",
@@ -2070,11 +2077,15 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "              width (default 80) can adjust line number computation",
 "              to the output terminal's line width.",
 #ifdef Xorriso_with_readlinE
-"  -use_stdin  Use raw standard input even if libreadline is available",
-"  -use_readline  Use libreadline for dialog if available",
+"  -use_readline \"on\"|\"off\"",
+"              Whether to use libreadline for dialog if available.",
 "  -history text  Copy text into libreadline history. This command",
 "              itself is not copied to the history list.",
 #endif /* Xorriso_with_readlinE */
+"  -sh_style_result \"on\"|\"off\"",
+"              If \"on\" do not wrap file addresses in quotation marks with",
+"                -pwd -pwdx -ls -lsd -lsl -lsdl -lsx -lsdx -lslx -lsdlx",
+"                -du -dus -dux -dusx -findx -find",
 "  -backslash_codes \"on\"|\"off\"|",
 "                   \"in_double_quotes\"|\"in_quotes\"|\"with_quoted_input\"",
 "                   [:\"with_program_arguments\"][:\"encode_output\"]",
@@ -2090,9 +2101,17 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "                I:1: enter option text :",
 "              -pkt_output:on is intended for use by frontend programs.",
 "  -msg_op \"start_sieve\"|\"read_sieve\"|\"clear_sieve\"|\"end_sieve\"|",
-"          \"parse\"|\"parse_bulk\"|\"compare_sev\"|\"list_sev\" param_text",
+"          \"parse\"|\"parse_silently\"|\"parse_bulk\"|\"parse_bulk_silently\"|",
+"          \"compare_sev\"|\"list_sev\" param_text",
 "              Enable, use, or disable message sieve. Or parse lines into",
 "              words. Or compare or list severity names.",
+"  -named_pipes_loop mode[:mode]",
+"                    disk_path_stdin disk_path_stdout disk_path_stderr",
+"              Enter an EOF resistant dialog loop at a named pipe as input",
+"              and zero, one or two named pipes as output. \"-\" = no pipe.",
+"              Mode \"cleanup\" removes pipes at loop end. \"keep\" does not.",
+"              Mode \"buffered\" reads all lines from input pipe before it",
+"              opens the output pipes. \"direct\" opens after first input.",
 "  -launch_frontend program [args ...] --",
 "              Start a program, connect its stdin to xorriso stdout and",
 "              stderr, connect its stdout to xorriso stdin.",
@@ -2251,7 +2270,7 @@ ex:;
 /* Option -history */
 int Xorriso_option_history(struct XorrisO *xorriso, char *line, int flag)
 {
- Xorriso_dialog_input(xorriso,line,strlen(line)+1,2);
+ Xorriso_dialog_input(xorriso, line, strlen(line) + 1, 2 | 32);
  return(1);
 }
 
