@@ -68,7 +68,7 @@ struct XorrisO;
 */
 #define Xorriso_header_version_majoR  1
 #define Xorriso_header_version_minoR  3
-#define Xorriso_header_version_micrO  2
+#define Xorriso_header_version_micrO  4
 
 
 /** If needed: Something like ".pl01" to indicate a bug fix. Normally empty.
@@ -1353,7 +1353,7 @@ int Xorriso_option_chowni(struct XorrisO *xorriso, char *uid,
 int Xorriso_option_clone(struct XorrisO *xorriso, char *origin, char *dest,
                          int flag);
 
-/* Command -close "on"|"off" */
+/* Command -close "on"|"off"| @since 1.3.4 "as_needed" */
 int Xorriso_option_close(struct XorrisO *xorriso, char *mode, int flag);
 
 /* Command -close_damaged */
@@ -1871,7 +1871,10 @@ int Xorriso_option_signal_handling(struct XorrisO *xorriso, char *mode,
 /* @since 1.1.8 */
 int Xorriso_option_sleep(struct XorrisO *xorriso, char *duration, int flag);
 
-/* Command -speed */
+/* Command -speed , -read_speed */
+/* @param flag bit0= @since 1.3.4
+                     -read_speed rather than -speed
+*/ 
 int Xorriso_option_speed(struct XorrisO *xorriso, char *speed, int flag);
 
 /* Command -split_size */
