@@ -232,6 +232,8 @@ struct XorrisO { /* the global context of xorriso */
  char biblio_file[38];
  char abstract_file[38];
 
+ char application_use[SfileadrL];
+
  char session_logfile[SfileadrL];
  int session_lba;
  int session_blocks;
@@ -307,9 +309,12 @@ struct XorrisO { /* the global context of xorriso */
                             loaded image. */
 
  int volset_change_pending; /* whether -commit would make sense
-                               0= no change pending , 1= change pending
+                               0= no change pending
+                               1= change pending
                                2= change pending, but -as misofs -print-size
                                   was performed on the changed image model
+                               3= change pending, but the attempt to write it
+                                  failed
                              */
  int no_volset_present;     /* set to 1 on first failure */
 
@@ -336,7 +341,12 @@ struct XorrisO { /* the global context of xorriso */
  int ban_stdio_write;
  int do_dummy;
  int do_close;
- int speed;     /* in libburn units : 1000 bytes/second , 0 = Max, -1 = Min */
+ int auto_close;     /* Whether to let do_close depend on media state */
+
+ int write_speed;    /* Write speed in libburn units : 1000 bytes/second ,
+                                           0 = Max, -1 = Min, -2= do not set */
+ int read_speed;     /* Read speed. See above */
+
  int fs;        /* fifo size in 2048 byte chunks : at most 1 GB */
  int padding;   /* number of bytes to add after ISO 9660 image */
  int do_padding_by_libisofs; /* 0= by libburn , 1= by libisofs */
@@ -520,6 +530,8 @@ struct XorrisO { /* the global context of xorriso */
 
  int dialog;  /* 0=off , 1=single-line , 2=multi-line */
 
+ struct Xorriso_lsT *buffered_dialog; /* If not NULL : read by dialog */
+
  int bsl_interpretation;
                      /* whether to run input through Sfile_bsl_interpreter():
                         bit0-1= dialog and quoted file reading
@@ -533,6 +545,14 @@ struct XorrisO { /* the global context of xorriso */
                         bit6= perform backslash encoding with info texts
                      */
 
+ int sh_style_result; /* Whether not to wrap into quotation marks the file
+                         addresses reported by:
+                           pwd pwdx ls lsd lsl lsdl lsx lsdx lslx lsdlx
+                           du dus dux dusx findx find
+                         and to make du* numbers left adjusted,
+                         and not to append "/" to pwd* 
+                       */
+
  /* Pattern matching facility. It still carries legacy from scdbackup/askme.c
     but is fully functional for xorriso.
  */
@@ -572,6 +592,8 @@ struct XorrisO { /* the global context of xorriso */
  struct ExclusionS *hfsplus_hidings;
 
  int use_stdin; /* use raw stdin even if readline support is compiled */
+ int tolerate_stdin_eof;  /* Do not abort on EOF in Xorriso_dialog_input
+                             but rather return -2. */
  int result_page_length;
  int result_page_width;
  char mark_text[SfileadrL]; /* ( stdout+stderr, M: ) */
