@@ -19,7 +19,7 @@
 */
 #define xorriso_libisoburn_req_major  1
 #define xorriso_libisoburn_req_minor  3
-#define xorriso_libisoburn_req_micro  0
+#define xorriso_libisoburn_req_micro  4
 
 
 struct SpotlisT;          /* List of intervals with different read qualities */
@@ -45,6 +45,8 @@ int Xorriso_give_up_drive(struct XorrisO *xorriso, int flag);
 
 int Xorriso_write_session(struct XorrisO *xorriso, int flag);
 
+int Xorriso_retry_write_session(struct XorrisO *xorriso, int flag);
+
 /* @param boss_iter Opaque handle to be forwarded to actions in ISO image
                     Set to NULL if calling this function from outside ISO world
    @param flag bit0= mkdir: graft in as empty directory, not as copy from disk
@@ -233,6 +235,10 @@ int Xorriso_atip(struct XorrisO *xorriso, int flag);
 int Xorriso_burn_track(struct XorrisO *xorriso, off_t write_start_address,
                        char *track_source, off_t tsize, int flag);
 
+int Xorriso_retry_burn_track(struct XorrisO *xorriso,  
+                             off_t write_start_address,
+                             char *track_source, off_t tsize, int flag);
+
 /* @param flag bit1= outdev rather than indev
    @return <=0 = failure , 1= ok , 2= ok, is CD profile
 */ 
@@ -608,5 +614,9 @@ int Xorriso_hfsplus_bless(struct XorrisO *xorriso, char *path,
 
 int Xorriso_pretend_full_disc(struct XorrisO *xorriso, int flag);
 
+
+int Xorriso_apply_sort_file(struct XorrisO *xorriso, char *path, int flag);
+
+
 #endif /* Xorrisoburn_includeD */
 
